@@ -26,32 +26,51 @@
     THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef DISPLAY_H
-#define DISPLAY_H
+#ifndef SDLAPP_DISPLAY_H
+#define SDLAPP_DISPLAY_H
 
-#include <SDL/SDL.h>
-#include <SDL/SDL_opengl.h>
+#include "extensions.h"
 
 #include "logger.h"
 #include "vectors.h"
 
+#include "texture.h"
+#include "fxfont.h"
+
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 #include <vector>
 #include <string>
-#include <string.h>
 
-class Display {
+class SDLInitException : public std::exception {
+protected:
+    std::string error;
+public:
+    SDLInitException(std::string& error) : error(error) {}
+    virtual ~SDLInitException() throw () {};
+
+    virtual const char* what() const throw() { return error.c_str(); }
+};
+
+class SDLAppDisplay {
     SDL_Surface *surface;
 
+    bool enable_shaders;
+    bool enable_alpha;
+    bool vsync;
+
+    int  multi_sample;
+
     int    SDLFlags(bool fullscreen);
 public:
     int width, height;
     bool fullscreen;
     vec3f clearColour;
 
-    Display();
-    ~Display();
-
-    void   displayArgs(int argc, char *argv[], int* xres, int* yres, bool* fullscreen);
+    SDLAppDisplay();
+    ~SDLAppDisplay();
 
     void   init(std::string window_title, int xres, int yres, bool fullscreen);
 
@@ -59,16 +78,33 @@ public:
 
     void   update();
     void   clear();
+    void   enableVsync(bool vsync);
     void   setClearColour(vec3f colour);
 
+#ifdef SDLAPP_SHADER_SUPPORT
+    void   enableShaders(bool enable);
+#endif
+
+    void   enableAlpha(bool enable);
+    void   multiSample(int sample);
+
     void   mode3D(float fov, float znear, float zfar);
     void   mode2D();
 
     vec4f  currentColour();
 
-    GLuint createTexture(int width, int height, int mipmaps, int clamp, int trilinear, int format, unsigned int* data);
+    void fullScreenQuad(bool coord_flip);
+
+    void   renderToTexture(GLuint target, int width, int height, GLenum format);
+    GLuint emptyTexture(int width, int height, GLenum format);
+    GLuint createTexture(int width, int height, bool mipmaps, bool clamp, bool trilinear, GLenum format, unsigned int* data);
+
+    vec3f project(vec3f pos);
+    vec3f unproject(vec2f pos);
+
+    void checkGLErrors();
 };
 
-extern Display display;
+extern SDLAppDisplay display;
 
 #endif
