@@ -1,6 +1,5 @@
-
 /*
-    Copyright (c) 2008 Andrew Caudwell (acaudwell@gmail.com)
+    Copyright (c) 2009 Andrew Caudwell (acaudwell@gmail.com)
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -31,6 +30,22 @@
 
 #include <map>
 #include <string>
+#include <cstring>
+#include <stdexcept>
+
+#ifdef _WIN32
+#include "windows.h"
+#endif
+
+class ResourceException : public std::exception {
+protected:
+    std::string resource;
+public:
+    ResourceException(std::string& resource) : resource(resource) {}
+    virtual ~ResourceException() throw () {};
+
+    virtual const char* what() const throw() { return resource.c_str(); }
+};
 
 class Resource {
     int refs;
@@ -45,10 +60,18 @@ public:
 class ResourceManager {
 
 protected:
-    std::map<std::string, Resource*> resources;
+    std::map<std::string,Resource*> resources;
+    std::string resource_dir;
 public:
+
+    void setDir(std::string resource_dir);
+    std::string getDir();
+
     ResourceManager();
     virtual ~ResourceManager();
+
+    void purge();
+
     void release(Resource* resource);
     Resource* grab(std::string name);
     virtual Resource* create(std::string name) { return 0; };
