@@ -1,6 +1,5 @@
-
 /*
-    Copyright (c) 2008 Andrew Caudwell (acaudwell@gmail.com)
+    Copyright (c) 2009 Andrew Caudwell (acaudwell@gmail.com)
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -25,7 +24,7 @@
     (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
     THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
- 
+
 #include "texture.h"
 
 TextureManager texturemanager;
@@ -35,29 +34,45 @@ TextureManager texturemanager;
 TextureManager::TextureManager() : ResourceManager() {
 }
 
-TextureResource* TextureManager::grab(std::string name, int mipmaps, int clamp, int trilinear) {
+TextureResource* TextureManager::grab(std::string name, int mipmaps, int clamp, int trilinear, bool external_file) {
     //debugLog("grabing %s\n", name.c_str());
     Resource* r = resources[name];
 
     if(r==0) {
         //debugLog("%s not found. creating resource...\n", name.c_str());
-        r = create(name, mipmaps, clamp, trilinear);
+
+        r = new TextureResource(name, mipmaps, clamp, trilinear, external_file);
+
         resources[name] = r;
     }
     r->addref();
     return (TextureResource*)r;
 }
 
-
-TextureResource* TextureManager::create(std::string name, int mipmaps, int clamp, int trilinear) {
-    TextureResource* tr = new TextureResource(name, mipmaps, clamp, trilinear);
-    return tr;
-}
-
 // texture resource
 
-TextureResource::TextureResource(std::string name, int mipmaps, int clamp, int trilinear) : Resource(name) {
-    loadTexture(name, mipmaps, clamp, trilinear);
+TextureResource::TextureResource(std::string file, int mipmaps, int clamp, int trilinear, bool external_file) : Resource(file) {
+
+    //if doesnt have an absolute path, look in resource dir
+    if(!external_file && !(file.size() > 2 && file[1] == ':') && !(file.size() > 1 && file[0] == '/')) {
+        file = texturemanager.getDir() + file;
+    }
+
+    debugLog("creating texture from %s\n", file.c_str());
+
+    SDL_Surface *surface = IMG_Load(file.c_str());
+
+    if(surface==0) throw TextureException(file);
+
+    w = surface->w;
+    h = surface->h;
+
+    //figure out image colour order
+    int format = colourFormat(surface);
+
+    if(format==0) throw TextureException(file);
+
+    textureid = display.createTexture(w, h, mipmaps, clamp, trilinear, format, (unsigned int*) surface->pixels);
 }
 
 int TextureResource::colourFormat(SDL_Surface* surface) {
@@ -81,31 +96,6 @@ int TextureResource::colourFormat(SDL_Surface* surface) {
     return format;
 }
 
-void TextureResource::loadTexture(std::string file, int mipmaps, int clamp, int trilinear) {
-    debugLog("creating texture from %s\n", file.c_str());
-
-    SDL_Surface *surface = IMG_Load(file.c_str());
-
-    if(surface==0) {
-        printf("failed to load image %s", file.c_str());
-        exit(1);
-    }
-
-    w = surface->w;
-    h = surface->h;
-
-    //figure out image colour order
-    int format = colourFormat(surface);
-
-    if(format==0) {
-        printf("image %s has unsupported colour format", file.c_str());
-        exit(1);
-    }
-
-    textureid = display.createTexture(w, h, mipmaps, clamp, trilinear, format, (unsigned int*) surface->pixels);
-}
-
-
 TextureResource::~TextureResource() {
     if(textureid!=0) glDeleteTextures(1, &textureid);
 }
