@@ -20,17 +20,23 @@
 //Logstalgia
 
 //turn performance profiling
-//#DEFINE LS_PERFORMANCE_PROFILE
+//#define LS_PERFORMANCE_PROFILE
 
 float gSplash = -1.0f;
 float gStartPosition = 0.0;
 float gStopPosition  = 1.0;
-
+float gPaddlePosition = 0.67;
+bool  gAutoSkip = true;
+int   gFontSize = 14;
 bool  gDisableProgress = false;
+bool  gSyncLog         = false;
+bool  gHideURLPrefix   = false;
 
 std::string profile_name;
 Uint32 profile_start_msec;
 
+std::string old_tz;
+
 void profile_start(std::string profile) {
 #ifdef LS_PERFORMANCE_PROFILE
     profile_start_msec = SDL_GetTicks();
@@ -73,23 +79,26 @@ void logstalgia_help() {
     printf("  -x --full-hostnames        Show full request ip/hostname\n");
     printf("  -s --speed                 Simulation speed (default: 1)\n");
     printf("  -u --update-rate           Page summary update rate (default: 5)\n\n");
-
     printf("  -g name,regex,percent[,colour]  Group urls that match a regular expression\n\n");
 
-    printf("  --paddle-mode MODE         Paddle mode (single, pid, vhost)\n\n");
+    printf("  --paddle-mode MODE         Paddle mode (single, pid, vhost)\n");
+    printf("  --paddle-position POSITION Paddle position as a fraction of the view width\n\n");
 
-    printf("  --sync                     Begin with the next entry received (implies STDIN)\n");
+    printf("  --sync                     Read from STDIN, ignoring entries before now\n");
     printf("  --start-position POSITION  Begin at some position in the log (0.0 - 1.0)\n");
     printf("  --stop-position  POSITION  Stop at some position\n\n");
 
     printf("  --no-bounce                No bouncing\n\n");
 
     printf("  --hide-response-code       Hide response code\n");
-    printf("  --hide-paddle              Hide paddle\n\n");
-
+    printf("  --hide-paddle              Hide paddle\n");
+    printf("  --hide-url-prefix          Hide URL protocol and hostname prefix\n\n");
+    printf("  --disable-auto-skip        Disable skipping of empty time periods\n");
     printf("  --disable-progress         Disable the progress bar\n");
     printf("  --disable-glow             Disable the glow effect\n\n");
 
+    printf("  --font-size SIZE           Font size\n\n");
+
     printf("  --glow-duration            Duration of the glow (default: 0.15)\n");
     printf("  --glow-multiplier          Adjust the amount of glow (default: 1.25)\n");
     printf("  --glow-intensity           Intensity of the glow (default: 0.5)\n\n");
@@ -112,7 +121,6 @@ Logstalgia::Logstalgia(std::string logfile, float simu_speed, float update_rate)
     paused     = false;
     recentre   = false;
     next       = false;
-    sync       = false;
 
     this->simu_speed  = simu_speed;
     this->update_rate = update_rate;
@@ -121,12 +129,13 @@ Logstalgia::Logstalgia(std::string logfile, float simu_speed, float update_rate)
 
     spawn_delay=0;
 
-    gHighscore = 0;
+    highscore = 0;
 
     uimessage_timer=0.0f;
 
     ipSummarizer  = 0;
 
+    mintime       = gSyncLog ? time(0) : 0;
     seeklog       = 0;
     streamlog     = 0;
 
@@ -139,9 +148,6 @@ Logstalgia::Logstalgia(std::string logfile, float simu_speed, float update_rate)
         streamlog = new StreamLog();
         gDisableProgress = true;
 
-        //buffer STDIN
-        buffer_row_count = 500;
-
     } else {
         try {
             seeklog = new SeekLog(logfile);
@@ -149,9 +155,6 @@ Logstalgia::Logstalgia(std::string logfile, float simu_speed, float update_rate)
         } catch(SeekLogException& exception) {
             throw SDLAppException("unable to read log file");
         }
-
-        //dont buffer
-        buffer_row_count = 1;
     }
 
     total_space = display.height - 40;
@@ -163,7 +166,8 @@ Logstalgia::Logstalgia(std::string logfile, float simu_speed, float update_rate)
 
     fontLarge  = fontmanager.grab("FreeSerif.ttf", 42);
     fontMedium = fontmanager.grab("FreeMonoBold.ttf", 16);
-    fontSmall  = fontmanager.grab("FreeMonoBold.ttf", 14);
+    fontBall   = fontmanager.grab("FreeMonoBold.ttf", 16);
+    fontSmall  = fontmanager.grab("FreeMonoBold.ttf", gFontSize);
 
     fontLarge.dropShadow(true);
     fontMedium.dropShadow(true);
@@ -194,10 +198,21 @@ Logstalgia::Logstalgia(std::string logfile, float simu_speed, float update_rate)
     screen_blank_period   = 60.0;
     screen_blank_elapsed  = 0.0;
 
+    paddle_x = display.width * gPaddlePosition;
     paddle_colour = (gPaddleMode > PADDLE_SINGLE) ?
         vec4f(0.0f, 0.0f, 0.0f, 0.0f) : vec4f(0.5, 0.5, 0.5, 1.0);
 
     debugLog("Logstalgia end of constructor\n");
+
+    //check if TZ is set, store current value
+    if(old_tz.empty()) {
+        char* current_tz_env = getenv("TZ");
+
+        if(current_tz_env != 0) {
+            old_tz  = std::string("TZ=");
+            old_tz += std::string(current_tz_env);
+        }
+    }
 }
 
 Logstalgia::~Logstalgia() {
@@ -218,11 +233,6 @@ Logstalgia::~Logstalgia() {
 
 }
 
-void Logstalgia::syncLog() {
-    if(streamlog == 0) return;
-    sync = true;
-}
-
 void Logstalgia::togglePause() {
     paused = !paused;
 
@@ -237,10 +247,10 @@ void Logstalgia::togglePause() {
 }
 
 void Logstalgia::keyPress(SDL_KeyboardEvent *e) {
-	if (e->type == SDL_KEYDOWN) {
+    if (e->type == SDL_KEYDOWN) {
 
-		if (e->keysym.sym == SDLK_ESCAPE) {
-		    appFinished=true;
+        if (e->keysym.sym == SDLK_ESCAPE) {
+            appFinished=true;
         }
 
         if(e->keysym.sym == SDLK_q) {
@@ -248,7 +258,7 @@ void Logstalgia::keyPress(SDL_KeyboardEvent *e) {
         }
 
         if(e->keysym.sym == SDLK_c) {
-            gSplash = 15.0f;
+            gSplash = 10.0f;
         }
 
         if(e->keysym.sym == SDLK_n) {
@@ -291,12 +301,14 @@ void Logstalgia::keyPress(SDL_KeyboardEvent *e) {
             }
         }
 
-	}
+    }
 }
 
 void Logstalgia::reset() {
 
-    gHighscore = 0;
+    end_reached = false;
+
+    highscore = 0;
 
     for(std::map<std::string, Paddle*>::iterator it= paddles.begin(); it!=paddles.end();it++) {
         delete it->second;
@@ -304,7 +316,7 @@ void Logstalgia::reset() {
     paddles.clear();
 
     if(gPaddleMode <= PADDLE_SINGLE) {
-        vec2f paddle_pos = vec2f(display.width-(display.width/3), rand() % display.height);
+        vec2f paddle_pos = vec2f(paddle_x - 20, rand() % display.height);
         Paddle* paddle = new Paddle(paddle_pos, paddle_colour, "");
         paddles[""] = paddle;
     }
@@ -321,7 +333,7 @@ void Logstalgia::reset() {
         summGroups[i]->recalc_display();
     }
 
-    entries.clear();
+    queued_entries.clear();
 
     // reset settings
     elapsed_time  = 0;
@@ -406,7 +418,8 @@ void Logstalgia::mouseMove(SDL_MouseMotionEvent *e) {
 
 Regex ls_url_hostname_regex("^http://[^/]+(.+)$");
 
-std::string Logstalgia::filterURLHostname(std::string hostname) {
+std::string Logstalgia::filterURLHostname(const std::string& hostname) {
+
     std::vector<std::string> matches;
 
     if(ls_url_hostname_regex.match(hostname, &matches)) {
@@ -416,17 +429,38 @@ std::string Logstalgia::filterURLHostname(std::string hostname) {
     return hostname;
 }
 
-void Logstalgia::addBall(LogEntry& le, float head_start) {
-    debugLog("adding ball for log entry\n");
+void Logstalgia::addStrings(LogEntry* le) {
 
-    gHighscore++;
+    std::string hostname = le->hostname;
+    std::string pageurl  = le->path;
 
-    std::string hostname = le.hostname;
-    std::string pageurl  = le.path;
+    int nogroups = summGroups.size();
+    Summarizer* pageSummarizer= 0;
+
+    for(int i=0;i<nogroups;i++) {
+        if(summGroups[i]->supportedString(pageurl)) {
+            pageSummarizer = summGroups[i];
+            break;
+        }
+    }
+
+    if(pageSummarizer==0) return;
+
+    if(gHideURLPrefix) pageurl = filterURLHostname(pageurl);
+
+    pageSummarizer->addString(pageurl);
+    ipSummarizer->addString(hostname);
+}
+
+void Logstalgia::addBall(LogEntry* le, float start_offset) {
+
+    std::string hostname = le->hostname;
+    std::string pageurl  = le->path;
 
     //find appropriate summarizer for url
     int nogroups = summGroups.size();
     Summarizer* pageSummarizer= 0;
+
     for(int i=0;i<nogroups;i++) {
         if(summGroups[i]->supportedString(pageurl)) {
             pageSummarizer = summGroups[i];
@@ -440,7 +474,7 @@ void Logstalgia::addBall(LogEntry& le, float head_start) {
 
     if(gPaddleMode > PADDLE_SINGLE) {
 
-        std::string paddle_token = (gPaddleMode == PADDLE_VHOST) ? le.vhost : le.pid;
+        std::string paddle_token = (gPaddleMode == PADDLE_VHOST) ? le->vhost : le->pid;
 
         entry_paddle = paddles[paddle_token];
 
@@ -454,26 +488,25 @@ void Logstalgia::addBall(LogEntry& le, float head_start) {
         entry_paddle = paddles[""];
     }
 
+    if(gHideURLPrefix) pageurl = filterURLHostname(pageurl);
 
-    pageurl = filterURLHostname(pageurl);
+    float dest_y = pageSummarizer->getMiddlePosY(pageurl);
+    float pos_y  = ipSummarizer->getMiddlePosY(hostname);
 
-    float dest_y = pageSummarizer->addString(pageurl);
+    float start_x = -(entry_paddle->getX()/ 5.0f);
 
-    float pos_y  = ipSummarizer->addString(hostname);
-
-    vec2f pos  = vec2f(1, pos_y);
-    vec2f dest = vec2f(entry_paddle->getX(), dest_y);
-
-    std::string match = ipSummarizer->getBestMatchStr(hostname);
+    vec2f ball_start = vec2f(start_x, pos_y);
+    vec2f ball_dest  = vec2f(entry_paddle->getX(), dest_y);
 
+    const std::string& match = ipSummarizer->getBestMatchStr(hostname);
 
     vec3f colour = pageSummarizer->isColoured() ? pageSummarizer->getColour() : colourHash(match);
 
-    RequestBall* ball = new RequestBall(le, fontMedium, balltex, colour, pos, dest, simu_speed);
-    ball->setElapsed(head_start);
+    RequestBall* ball = new RequestBall(le, &fontMedium, balltex, colour, ball_start, ball_dest, simu_speed);
+
+    ball->setElapsed( start_offset );
 
     balls.push_back(ball);
-    spawn_delay=spawn_speed;
 }
 
 BaseLog* Logstalgia::getLog() {
@@ -482,22 +515,27 @@ BaseLog* Logstalgia::getLog() {
     return streamlog;
 }
 
-std::string whitespaces (" \t\f\v\n\r");
+void Logstalgia::readLog(int buffer_rows) {
 
-void Logstalgia::readLog() {
-    debugLog("readLog()\n");
+    profile_start("readLog");
 
-    int lineno=0;
+    //change TZ to UTC
+    putenv((char*)"TZ=UTC");
+    tzset();
+
+    int entries_read = 0;
 
     std::string linestr;
     BaseLog* baselog = getLog();
 
+    time_t read_timestamp = 0;
+
     while( baselog->getNextLine(linestr) ) {
 
         //trim whitespace
         if(linestr.size()>0) {
             size_t string_end =
-                linestr.find_last_not_of(whitespaces);
+                linestr.find_last_not_of(" \t\f\v\n\r");
 
             if(string_end == std::string::npos) {
                 linestr = "";
@@ -508,26 +546,24 @@ void Logstalgia::readLog() {
 
         LogEntry le;
 
+        bool parsed_entry;
+
         //determine format
         if(accesslog==0) {
 
-            //is this an apache access log?
-            ApacheLog* apachelog = new ApacheLog();
-            if(apachelog->parseLine(linestr, le)) {
-                accesslog = apachelog;
-                entries.push_back(le);
-                total_entries++;
+            //is this a recognized NCSA access log?
+            NCSALog* ncsalog = new NCSALog();
+            if((parsed_entry = ncsalog->parseLine(linestr, le))) {
+                accesslog = ncsalog;
             } else {
-                delete apachelog;
+                delete ncsalog;
             }
 
             if(accesslog==0) {
                 //is this a custom log?
                 CustomAccessLog* customlog = new CustomAccessLog();
-                if(customlog->parseLine(linestr, le)) {
+                if((parsed_entry = customlog->parseLine(linestr, le))) {
                     accesslog = customlog;
-                    entries.push_back(le);
-                    total_entries++;
                 } else {
                     delete customlog;
                 }
@@ -535,28 +571,58 @@ void Logstalgia::readLog() {
 
         } else {
 
-            if(accesslog->parseLine(linestr, le)) {
-                entries.push_back(le);
-                total_entries++;
-            } else {
+            if(!(parsed_entry = accesslog->parseLine(linestr, le))) {
                 debugLog("error: could not read line %s\n", linestr.c_str());
             }
-
         }
 
-        lineno++;
+        if(parsed_entry) {
 
-        if(lineno>=buffer_row_count) break;
+            if(mintime == 0 || mintime <= le.timestamp) {
+
+                queued_entries.push_back(new LogEntry(le));
+
+                total_entries++;
+                entries_read++;
+
+                //read at least the buffered row count if specified
+                //otherwise read all entries with the same time
+                if(buffer_rows) {
+                    if(entries_read > buffer_rows) break;
+                } else {
+                    if(read_timestamp && read_timestamp < le.timestamp) break;
+                }
+
+                read_timestamp = le.timestamp;
+            }
+        }
     }
 
-    if(entries.size()==0 && seeklog != 0) {
+    profile_stop();
+
+    //reset TZ to previous value
+
+    if(!old_tz.empty()) {
+        putenv((char*)old_tz.c_str());
+    } else {
+#ifdef HAVE_UNSETENV
+        unsetenv("TZ");
+#else
+        putenv("TZ=");
+#endif
+    }
+
+    tzset();
+
+    if(queued_entries.empty() && seeklog != 0) {
 
         if(total_entries==0) {
             logstalgia_quit("could not parse first entry");
         }
 
         //no more entries
-        appFinished=true;
+        end_reached = true;
+
         return;
     }
 
@@ -564,19 +630,18 @@ void Logstalgia::readLog() {
         float percent = seeklog->getPercent();
 
         if(percent > gStopPosition) {
-            appFinished = true;
+            end_reached = true;
             return;
         }
 
-
         if(!gDisableProgress) slider.setPercent(percent);
     }
 
     //set start time if currently 0
-    if(starttime==0 && entries.size())
-        starttime = entries[0].timestamp;
-
-    debugLog("end of readLog()\n");
+    if(starttime==0 && !queued_entries.empty()) {
+        starttime = queued_entries.front()->timestamp;
+        currtime  = 0;
+    }
 }
 
 void Logstalgia::init() {
@@ -607,16 +672,6 @@ void Logstalgia::init() {
     if(gStartPosition > 0.0 && gStartPosition < 1.0) {
         seekTo(gStartPosition);
     }
-
-    if(sync && streamlog != 0) {
-        streamlog->consume();
-        entries.clear();
-
-        starttime     = time(0);
-        elapsed_time  = 0;
-        lasttime      = 0;
-    }
-
 }
 
 void Logstalgia::setBackground(vec3f background) {
@@ -667,7 +722,7 @@ void Logstalgia::update(float t, float dt) {
    framecount++;
 }
 
-RequestBall* Logstalgia::findNearest(Paddle* paddle, std::string paddle_token) {
+RequestBall* Logstalgia::findNearest(Paddle* paddle, const std::string& paddle_token) {
 
     float min_dist = -1.0f;
     RequestBall* nearest = 0;
@@ -676,14 +731,14 @@ RequestBall* Logstalgia::findNearest(Paddle* paddle, std::string paddle_token) {
         RequestBall* ball = *it;
 
         //special case if failed response code
-        if(!ball->le.successful) {
+        if(!ball->le->successful) {
             continue;
         }
 
-        if(ball->le.successful && !ball->bounced()
-            && (gPaddleMode <= PADDLE_SINGLE 
-                || gPaddleMode == PADDLE_VHOST && ball->le.vhost == paddle_token
-                || gPaddleMode == PADDLE_PID   && ball->le.pid   == paddle_token
+        if(ball->le->successful && !ball->hasBounced()
+            && (gPaddleMode <= PADDLE_SINGLE
+                || gPaddleMode == PADDLE_VHOST && ball->le->vhost == paddle_token
+                || gPaddleMode == PADDLE_PID   && ball->le->pid   == paddle_token
                )
             ) {
             float dist = (paddle->getX() - ball->getX())/ball->speed;
@@ -698,15 +753,15 @@ RequestBall* Logstalgia::findNearest(Paddle* paddle, std::string paddle_token) {
 }
 
 void Logstalgia::removeBall(RequestBall* ball) {
-    std::string url  = ball->le.path;
-    std::string host = ball->le.hostname;
+    std::string url  = ball->le->path;
+    std::string host = ball->le->hostname;
 
     int nogroups = summGroups.size();
 
     for(int i=0;i<nogroups;i++) {
         if(summGroups[i]->supportedString(url)) {
 
-            url = filterURLHostname(url);
+            if(gHideURLPrefix) url = filterURLHostname(url);
 
             summGroups[i]->removeString(url);
             break;
@@ -732,6 +787,11 @@ void Logstalgia::logic(float t, float dt) {
 
     infowindow.hide();
 
+    if(end_reached && balls.empty()) {
+        appFinished = true;
+        return;
+    }
+
     //if paused, dont move anything, only check what is under mouse
     if(paused) {
 
@@ -765,78 +825,110 @@ void Logstalgia::logic(float t, float dt) {
     elapsed_time += sdt;
     currtime = starttime + (long)(elapsed_time);
 
-    //next will fast forward clock to the time of the next entry, if the next entry is in the future
-    if(next || sync) {
-        if(entries.size() > 0) {
-            LogEntry le = entries[0];
+    //next will fast forward clock to the time of the next entry, 
+    //if the next entry is in the future
+    if(next || gAutoSkip && balls.empty()) {
+        if(!queued_entries.empty()) {
+            LogEntry* le = queued_entries.front();
 
-            long entrytime = le.timestamp;
+            long entrytime = le->timestamp;
             if(entrytime > currtime) {
                 elapsed_time = entrytime - starttime;
                 currtime = starttime + (long)(elapsed_time);
             }
-            sync = false;
         }
         next = false;
     }
 
     //recalc spawn speed each second by
     if(currtime != lasttime) {
-        profile_start("calc spawn speed");
-        int items_to_spawn=-1;
-        for(std::deque<LogEntry>::iterator it = entries.begin(); it != entries.end(); it++) {
-            if((*it).timestamp <= currtime) {
-                items_to_spawn++;
-                continue;
+
+        //dont bother reading the log if we dont need to
+        if(queued_entries.empty() || queued_entries.back()->timestamp <= currtime) {
+            readLog();
+        }
+
+        profile_start("determine new entries");
+
+        int items_to_spawn=0;
+
+        for(std::list<LogEntry*>::iterator it = queued_entries.begin(); it != queued_entries.end(); it++) {
+            LogEntry* le = *it;
+
+            if(le->timestamp > currtime) break;
+
+            items_to_spawn++;
+
+            addStrings(le);
+        }
+
+        profile_stop();
+
+        //debugLog("items to spawn %d\n", items_to_spawn);
+
+        if(items_to_spawn > 0) {
+
+            profile_start("add new strings");
+
+            //re-summarize
+            ipSummarizer->summarize();
+
+            int nogrps = summGroups.size();
+
+            for(int i=0;i<nogrps;i++) {
+                summGroups[i]->summarize();
             }
-            break;
+
+            profile_stop();
+
+            profile_start("add new entries");
+
+            float item_offset = 1.0 / (float) (items_to_spawn);
+
+            int item_no = 0;
+            
+            while(!queued_entries.empty()) {
+
+                LogEntry* le = queued_entries.front();
+
+                if(le->timestamp > currtime) break;
+
+                float pos_offset   = 1.0 - item_offset * (float) item_no++;
+                float start_offset = std::min(1.0f, pos_offset);
+
+		addBall(le, start_offset);
+
+                queued_entries.pop_front();
+            }
+
         }
 
-        spawn_speed = (items_to_spawn <= 0) ? 0.1f/simu_speed : (1.0f / items_to_spawn) / simu_speed;
-        spawn_delay = 0.0f;
-        debugLog("spawn_speed = %.2f\n", spawn_speed);
-        profile_stop();
+        //update date
+        if(total_entries>0) {
+            char datestr[256];
+            char timestr[256];
 
-        //display date
-        char datestr[256];
-        char timestr[256];
-        struct tm* timeinfo = localtime ( &currtime );
-        strftime(datestr, 256, "%A, %B %d, %Y", timeinfo);
-        strftime(timestr, 256, "%X", timeinfo);
+            struct tm* timeinfo = localtime ( &currtime );
+            strftime(datestr, 256, "%A, %B %d, %Y", timeinfo);
+            strftime(timestr, 256, "%X", timeinfo);
 
-        displaydate =datestr;
-        displaytime =timestr;
-    }
-
-    lasttime=currtime;
-
-    //see if entries show appear
-    if(entries.size() > 0 && spawn_delay<=0) {
-        profile_start("add entries");
-
-        LogEntry le = entries[0];
-        if(le.timestamp < currtime) {
-            addBall(le, -spawn_delay);
-
-            entries.pop_front();
+            displaydate = datestr;
+            displaytime = timestr;
+        } else {
+            displaydate = "";
+            displaytime = "";
         }
 
-        profile_stop();
-    }
-
-    //read log if we run out
-    if(entries.size()==0) {
-        profile_start("readLog");
-
-        readLog();
-        if(appFinished) return;
-
+        lasttime=currtime;
 
         profile_stop();
+    } else {
+        //do small reads per frame if we havent buffered the next second
+        if(queued_entries.empty() || queued_entries.back()->timestamp <= currtime+1) {
+            readLog(50);
+        }
     }
 
-    spawn_delay -= sdt;
-
     std::list<Paddle*> inactivePaddles;
 
     //update paddles
@@ -854,8 +946,8 @@ void Logstalgia::logic(float t, float dt) {
 
                 RequestBall* ball = *bit;
 
-                if(   gPaddleMode == PADDLE_VHOST && ball->le.vhost == paddle_token
-                   || gPaddleMode == PADDLE_PID   && ball->le.pid   == paddle_token) {
+                if(   gPaddleMode == PADDLE_VHOST && ball->le->vhost == paddle_token
+                   || gPaddleMode == PADDLE_PID   && ball->le->pid   == paddle_token) {
                     token_match = true;
                     break;
                 }
@@ -898,9 +990,9 @@ void Logstalgia::logic(float t, float dt) {
 
         RequestBall* ball = *it;
 
-        ball->logic(dt);
+        highscore += ball->logic(dt);
 
-        if(ball->finished()) {
+        if(ball->isFinished()) {
             it = balls.erase(it);
             removeBall(ball);
         } else {
@@ -975,15 +1067,13 @@ void Logstalgia::addGroup(std::string grouptitle, std::string groupregex, int pe
 
     int space = (int) ( ((float)percent/100) * total_space );
 
-    float pos_x = display.width-(display.width/3) + 20;
-
     int top_gap    = total_space - remaining_space;
     int bottom_gap = display.height - (total_space - remaining_space + space);
 
-    debugLog("add group name = %s, regex = %s, remainpc = %d, space = %d, pos_x = %.2f, top_gap = %d, bottom_gap = %d\n",
-        grouptitle.c_str(), groupregex.c_str(), remainpc, space, pos_x, top_gap, bottom_gap);
+    //debugLog("group %s: regex = %s, remainpc = %d, space = %d, top_gap = %d, bottom_gap = %d\n",
+    //    grouptitle.c_str(), groupregex.c_str(), remainpc, space, top_gap, bottom_gap);
 
-    Summarizer* summ = new Summarizer(fontSmall, pos_x, top_gap, bottom_gap, update_rate, groupregex, grouptitle);
+    Summarizer* summ = new Summarizer(fontSmall, paddle_x, top_gap, bottom_gap, update_rate, groupregex, grouptitle);
 //    summ->showCount(true);
 
     if(colour.length2() > 0.01f) {
@@ -1045,7 +1135,10 @@ void Logstalgia::draw(float t, float dt) {
     profile_start("draw balls");
 
     glEnable(GL_BLEND);
-    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    glEnable(GL_TEXTURE_2D);
+
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    glBindTexture(GL_TEXTURE_2D, balltex->textureid);
 
     for(std::list<RequestBall*>::iterator it = balls.begin(); it != balls.end(); it++) {
         (*it)->draw(dt);
@@ -1053,7 +1146,17 @@ void Logstalgia::draw(float t, float dt) {
 
     profile_stop();
 
-    profile_start("draw paddle(s)");
+    profile_start("draw response codes");
+
+    for(std::list<RequestBall*>::iterator it = balls.begin(); it != balls.end(); it++) {
+        RequestBall* r = *it;
+
+        if(gResponseCode && r->hasBounced()) {
+            r->drawResponseCode();
+        }
+    }
+
+    profile_stop();
 
     glDisable(GL_TEXTURE_2D);
     glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -1073,8 +1176,6 @@ void Logstalgia::draw(float t, float dt) {
 
     }
 
-    profile_stop();
-
     if(!gDisableGlow) {
 
         glBlendFunc (GL_ONE, GL_ONE);
@@ -1106,15 +1207,18 @@ void Logstalgia::draw(float t, float dt) {
 
     if(gSplash>0.0f) {
         int logowidth = fontLarge.getWidth("Logstalgia");
-        int logoheight = 100;
+        int logoheight = 105;
         int cwidth    = fontMedium.getWidth("Website Access Log Viewer");
-        int awidth    = fontSmall.getWidth("(C) 2008 Andrew Caudwell");
+        int awidth    = fontMedium.getWidth("(C) 2008 Andrew Caudwell");
 
         vec2f corner(display.width/2 - logowidth/2 - 30.0f,
                      display.height/2 - 45);
 
+        float logo_alpha = std::min(1.0f, gSplash/3.0f);
+        float logo_bg    = std::min(0.2f, gSplash/10.0f);
+
         glDisable(GL_TEXTURE_2D);
-        glColor4f(0.0f, 0.5f, 1.0f, font_alpha * gSplash * 0.015f);
+        glColor4f(0.0f, 0.5f, 1.0f, logo_bg);
         glBegin(GL_QUADS);
             glVertex2f(0.0f,                 corner.y);
             glVertex2f(0.0f,                 corner.y + logoheight);
@@ -1127,14 +1231,14 @@ void Logstalgia::draw(float t, float dt) {
         fontLarge.alignTop(true);
         fontLarge.dropShadow(true);
 
-        glColor4f(1,1,1,font_alpha);
+        glColor4f(1,1,1,logo_alpha);
         fontLarge.draw(display.width/2 - logowidth/2,display.height/2 - 30, "Logstalgia");
-        glColor4f(0,1,1,font_alpha);
+        glColor4f(0,1,1,logo_alpha);
         fontLarge.draw(display.width/2 - logowidth/2,display.height/2 - 30, "Log");
 
-        glColor4f(1,1,1,font_alpha);
+        glColor4f(1,1,1,logo_alpha);
         fontMedium.draw(display.width/2 - cwidth/2,display.height/2 + 17, "Website Access Log Viewer");
-        fontSmall.draw(display.width/2 - awidth/2,display.height/2 + 37, "(C) 2008 Andrew Caudwell");
+        fontMedium.draw(display.width/2 - awidth/2,display.height/2 + 37, "(C) 2008 Andrew Caudwell");
 
         gSplash-=dt;
     }
@@ -1146,7 +1250,8 @@ void Logstalgia::draw(float t, float dt) {
     if(info) {
         fontMedium.print(2,2, "FPS %d", (int) fps);
         fontMedium.print(2,19,"Balls %03d", balls.size());
-        fontMedium.print(2,49,"Paddles %03d", paddles.size());
+        fontMedium.print(2,36,"Queue %03d", queued_entries.size());
+        fontMedium.print(2,53,"Paddles %03d", paddles.size());
     } else {
         fontMedium.draw(2,2,  displaydate.c_str());
         fontMedium.draw(2,19, displaytime.c_str());
@@ -1157,5 +1262,5 @@ void Logstalgia::draw(float t, float dt) {
 
     fontLarge.alignTop(false);
 
-    fontLarge.print(display.width-10-counter_width,display.height-10, "%08d", gHighscore);
+    fontLarge.print(display.width-10-counter_width,display.height-10, "%08d", highscore);
 }
