@@ -18,7 +18,7 @@
 #ifndef LOGSTALGIA_H
 #define LOGSTALGIA_H
 
-#define LOGSTALGIA_VERSION "1.0.0"
+#define LOGSTALGIA_VERSION "1.0.3"
 
 #ifdef _WIN32
 #include "windows.h"
@@ -29,7 +29,7 @@
 #include "core/stringhash.h"
 #include "core/seeklog.h"
 
-#include "apache.h"
+#include "ncsa.h"
 #include "custom.h"
 #include "logentry.h"
 #include "paddle.h"
@@ -44,7 +44,6 @@
 #include <dirent.h>
 #include <string>
 #include <vector>
-#include <deque>
 #include <list>
 #include <map>
 #include <time.h>
@@ -53,11 +52,16 @@
 
 extern int   gHighscore;
 extern bool  gBounce;
+extern bool  gSyncLog;
 extern bool  gResponseCode;
 extern bool  gDisableProgress;
+extern bool  gHideURLPrefix;
 extern float gSplash;
 extern float gStartPosition;
 extern float gStopPosition;
+extern float gPaddlePosition;
+extern bool  gAutoSkip;
+extern int   gFontSize;
 
 void logstalgia_help();
 void logstalgia_info(std::string msg);
@@ -68,8 +72,6 @@ class Logstalgia : public SDLApp {
 
     std::map<std::string,Paddle*> paddles;
 
-    int buffer_row_count;
-
     std::string logfile;
 
     std::string displaydate;
@@ -80,10 +82,15 @@ class Logstalgia : public SDLApp {
     bool recentre;
     bool next;
     bool sync;
+    bool end_reached;
 
-    long starttime;
-    long currtime;
-    long lasttime;
+    int highscore;
+
+    time_t mintime;
+
+    time_t starttime;
+    time_t currtime;
+    time_t lasttime;
 
     float screen_blank_interval;
     float screen_blank_period;
@@ -108,6 +115,7 @@ class Logstalgia : public SDLApp {
 
     vec3f background;
     vec4f paddle_colour;
+    float paddle_x;
 
     TextureResource* balltex;
     TextureResource* glowtex;
@@ -119,6 +127,7 @@ class Logstalgia : public SDLApp {
     FXFont fontSmall;
     FXFont fontMedium;
     FXFont fontLarge;
+    FXFont fontBall;
 
     Summarizer* ipSummarizer;
 
@@ -131,7 +140,7 @@ class Logstalgia : public SDLApp {
     SeekLog* seeklog;
     StreamLog* streamlog;
 
-    std::deque<LogEntry> entries;
+    std::list<LogEntry*> queued_entries;
     std::list<RequestBall*> balls;
 
     TextArea infowindow;
@@ -144,18 +153,20 @@ class Logstalgia : public SDLApp {
     int frameskip;
     FrameExporter* frameExporter;
 
-    std::string filterURLHostname(std::string hostname);
+    std::string filterURLHostname(const std::string& hostname);
 
     std::string dateAtPosition(float percent);
     void seekTo(float percent);
 
-    void readLog();
+    void readLog(int buffer_rows = 0);
 
-    RequestBall* findNearest(Paddle* paddle, std::string paddle_proc);
+    RequestBall* findNearest(Paddle* paddle, const std::string& paddle_token);
     void updateGroups(float dt);
     void drawGroups(float dt, float alpha);
 
-    void addBall(LogEntry& le,  float head_start);
+    void addStrings(LogEntry* le);
+
+    void addBall(LogEntry* le,  float start_offset);
     void removeBall(RequestBall* ball);
     void addGroup(std::string grouptitle, std::string groupregex, int percent = 0, vec3f colour = vec3f(0.0f, 0.0f, 0.0f));
     void togglePause();
@@ -163,8 +174,9 @@ class Logstalgia : public SDLApp {
     BaseLog* getLog();
 
     void reset();
-	void logic(float t, float dt);
-	void draw(float t, float dt);
+
+    void logic(float t, float dt);
+    void draw(float t, float dt);
 public:
 	Logstalgia(std::string logfile, float simu_speed, float update_rate);
 	~Logstalgia();
@@ -173,8 +185,6 @@ public:
 
     void setFrameExporter(FrameExporter* exporter, int video_framerate);
 
-    void syncLog();
-
     void setBackground(vec3f background);
 
     //inherited methods
