@@ -52,7 +52,6 @@ int main(int argc, char *argv[]) {
     float simu_speed  = 1.0f;
     float update_rate = 5.0f;
     bool multisample  = false;
-    bool sync_log = false;
 
     vec3f background = vec3f(0.0, 0.0, 0.0);
 
@@ -87,6 +86,11 @@ int main(int argc, char *argv[]) {
             continue;
         }
 
+        if(args == "--hide-url-prefix") {
+            gHideURLPrefix = true;
+            continue;
+        }
+
         if(args == "-s" || args == "--speed") {
 
             if((i+1)>=arguments.size()) {
@@ -154,6 +158,36 @@ int main(int argc, char *argv[]) {
             continue;
         }
 
+        if(args == "--paddle-position") {
+
+            if((i+1)>=arguments.size()) {
+                logstalgia_quit("specify paddle-position (0.25 - 0.75)");
+            }
+
+            gPaddlePosition = atof(arguments[++i].c_str());
+
+            if(gPaddlePosition < 0.25f || gPaddlePosition > 0.75f) {
+                logstalgia_quit("paddle-position outside of range 0.25 - 0.75");
+            }
+
+            continue;
+        }
+
+        if(args == "--font-size") {
+
+            if((i+1)>=arguments.size()) {
+                logstalgia_quit("specify font-size (10 - 40)");
+            }
+
+            gFontSize = atoi(arguments[++i].c_str());
+
+            if(gFontSize < 10 || gFontSize > 40) {
+                logstalgia_quit("font-size outside of range 10 - 40");
+            }
+
+            continue;
+        }
+
         if(args == "-b" || args == "--background") {
 
             if((i+1)>=arguments.size()) {
@@ -222,6 +256,11 @@ int main(int argc, char *argv[]) {
             continue;
         }
 
+        //disable automatic skipping of empty time periods
+        if(args == "--disable-auto-skip") {
+            gAutoSkip = false;
+            continue;
+        }
 
         //disable progress bar
         if(args == "--disable-progress") {
@@ -318,7 +357,7 @@ int main(int argc, char *argv[]) {
         }
 
         if(args == "--sync") {
-            sync_log = true;
+            gSyncLog = true;
             if(!logfile.size()) logfile = "-";
             continue;
         }
@@ -348,7 +387,18 @@ int main(int argc, char *argv[]) {
 
     // wait for a character on the file handle if reading stdin
     if(logfile == "-") {
+
+#ifdef _WIN32
+        DWORD available_bytes;
+        HANDLE stdin_handle = GetStdHandle(STD_INPUT_HANDLE);
+
+        while(PeekNamedPipe(stdin_handle, 0, 0, 0,
+            &available_bytes, 0) && available_bytes==0 && !std::cin.fail()) {
+            SDL_Delay(100);
+        }
+#else
         while(std::cin.peek() == EOF && !std::cin.fail()) SDL_Delay(100);
+#endif
         std::cin.clear();
     }
 
@@ -396,8 +446,6 @@ int main(int argc, char *argv[]) {
 
         ls->setBackground(background);
 
-        if(sync_log) ls->syncLog();
-
         ls->run();
 
     } catch(ResourceException& exception) {
