@@ -22,21 +22,21 @@
 
 #include "core/display.h"
 #include "core/vectors.h"
-#include "core/fontpack.h"
+#include "core/fxfont.h"
 
 class TextArea {
 
     vec3f colour;
     vec2f corner;
     std::vector<std::string> content;
-    FPInstance font;
+    FXFont font;
     int rectwidth;
     int rectheight;
     bool visible;
 public:
     TextArea();
-    TextArea(FPInstance font);
-    TextArea(std::vector<std::string>& content, FPInstance font, vec3f colour);
+    TextArea(FXFont font);
+    TextArea(std::vector<std::string>& content, FXFont font, vec3f colour);
 
     void hide();
     void setText(std::vector<std::string>& content);
